<?php
/**
 * SkeletonCart
 * (c) 2017 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace FreeShipping;


class Postage extends \App\Postage
{
    protected $fileName;
    protected $config;

    public function __construct($table, $freePostagePrice)
    {
        parent::__construct($table, $freePostagePrice);

        $this->fileName = __DIR__ . "/Config.php";

        if (!is_readable($this->fileName)) {
            throw new \RuntimeException("ディスカウント設定が読み込めません。");
        }
        $this->config = include $this->fileName;

        if (!isset($this->config['itemIds']) || !is_array($this->config['itemIds'])) {
            throw new \RuntimeException("itemIdsが設定されていません。");
        }
        if (!isset($this->config['groupId'])) {
            throw new \RuntimeException("groupIdが設定されていません。");
        }
        if (!isset($this->config['applyAllItems'])) {
            throw new \RuntimeException("applyAllItemsが設定されていません。");
        }
    }

    public function itemIds()
    {
        return $this->config['itemIds'];
    }

    public function groupId()
    {
        return $this->config['groupId'];
    }

    public function applyAllItems()
    {
        return $this->config['applyAllItems'] ? true : false;
    }

    public function calculate(\App\Cart $cart, \App\Slip $slip)
    {
        $postage = parent::calculate($cart, $slip);

        if ($cart->count() == 0) {
            return 0;
        }

        $freeShippingItemCount = 0;
        foreach ($cart as $cartItem) {
            if (count($this->itemIds()) > 0) {
                if (is_int(array_search($cartItem->id(), $this->itemIds()))) {
                    $freeShippingItemCount++;
                }
            } elseif ($this->groupId()) {
                if ($this->groupId() == $cartItem->groupId()) {
                    $freeShippingItemCount++;
                }
            }
        }

        $discount = false;
        if ($freeShippingItemCount > 0) {
            if ($this->applyAllItems()) {
                $discount = true;
            } else {
                if ($cart->count() === $freeShippingItemCount) {
                    $discount = true;
                }
            }
        }

        if ($discount) {
            return 0;
        }

        return $postage;

    }
}